from __future__ import absolute_import
from stawp.module import Module

class Module(Module):

    menu_pages = None
    menu_html = None

    def interpret(self, page, builder):
        if 'menu' in builder.config:
            if self.menu_pages is None:
                self.menu_pages = []
                for menu in builder.config['menu']:
                    self.menu_pages.append({})

            if 'menuTitle' not in page.config:
                page.config['menuTitle'] = page.config['title']

            for index, menu in enumerate(builder.config['menu']):
                if page.web_path in menu:
                    self.menu_pages[index][page.web_path] = page

    def render(self, builder):
        if 'menu' in builder.config:
            if self.menu_html is None:
                self.menu_html = builder.read_template(name='menu')

            for index, menu in enumerate(builder.config['menu']):
                builder.config['menu[%d]' % index] = ''
                for path in menu:
                    if path in self.menu_pages[index]:
                        page = self.menu_pages[index][path]
                        builder.config['menu[%d]' % index] += builder.interpolate(self.menu_html, page.config)




from __future__ import absolute_import
import collections
from stawp.module import Module

class Module(Module):

    sitemap = None
    
    def interpret(self, page, builder):
        if self.sitemap == None:
            self.sitemap = collections.OrderedDict()
            for url in builder.config.get('sitemap_overrides', []):
                if url['loc'] is None:
                    url['loc'] = '/'
                self.sitemap[url['loc']] = url

        sitemap_path = page.src_path[2:]
        if len(sitemap_path) > 0:
            sitemap_path += '/'
        if page.src_path.count('/') == 1:
            priority = builder.config.get('sitemap_priority_toplevel', '0.7')
        else:
            priority = builder.config.get('sitemap_priority_default', '0.5')
        url = {
            'loc': builder.config.get('path', '/') + sitemap_path,
            'priority': priority,
        }
        
        if url['loc'] not in self.sitemap and url['loc'] not in builder.config.get('sitemap_ignore', []):
            self.sitemap[url['loc']] = url

    def render(self, builder):
        sitemap = '<?xml version="1.0" encoding="UTF-8"?>'+"\n"
        sitemap += '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'+"\n"
        if hasattr(self.sitemap, 'iteritems'):
            values = self.sitemap.iteritems()
        else:
            values = self.sitemap.items()
        for loc, url in values:
            sitemap += "\t<url>\n"
            sitemap += "\t\t<loc>"+builder.config['url']+url['loc'][1:]+"</loc>\n"
            sitemap += "\t\t<priority>"+str(url['priority'])+"</priority>\n"
            sitemap += "\t</url>\n"
        sitemap += '</urlset>'
        with open(builder.dist + '/../sitemap.xml', 'w') as file:
            file.write(sitemap)




from __future__ import absolute_import
import os
import sys
import yaml
from PIL import Image
from stawp.module import Module

class Module(Module):

    templates = {}

    rendered_galleries = None
    rendered_gallery = None

    def __init__(self, builder):
        for template in ['category', 'gallery', 'image']:
            self.templates[template] = builder.read_template(name=os.path.join('gallery', template))
        self.templates['body'] = builder.read_template(name='body')

    def interpret(self, page, builder):
        if 'menu' in builder.config:
            builder.config['bottomMenu'] = builder.config['menu'][1]
            if self.rendered_galleries is None:
                self.rendered_galleries = {}
                for page_name in builder.config['bottomMenu']:
                    self.rendered_galleries[page_name] = ''
        else:
            builder.config['bottomMenu'] = []

        page.config['categoryTitle'] = ''

        if 'images' in page.config:
            # If the page config has changed and should be written
            changed = False

            if builder.options['discover_images']:
                for child in page.children:
                    try:
                        Image.open(os.path.join(builder.src, child))
                        child = child.split('/')[-1]
                        found = False
                        for image in page.config['images']:
                            if child == image or isinstance(image, dict) and child == list(image.keys())[0]:
                                found = True
                                break

                        if not found:
                            page.config['images'].append(child)
                            page.raw_config['images'].append(child)
                            changed = True
                    except KeyboardInterrupt:
                        raise
                    except:
                        pass

            rendered_gallery = ''
            page.config['pageTitle'] = page.config['title']
            if 'image_prefix' in builder.config:
                base_prefix = builder.config['image_prefix']
                out_prefix = base_prefix + page.dir_name + '-'
            else:
                base_prefix = None
                out_prefix = ''

            if isinstance(page.config['images'][0], list):
                images = []
                for group in page.config['images']:
                    images += group
                    images.append({'n': None})
                del images[-1]
                page.config['images'] = images

            for image in page.config['images']:
                original = image
                title = None

                if isinstance(image, dict):
                    path = list(image.keys())[0]
                    if path == 'n':
                        rendered_gallery += '</div><div class="gallery">'
                    elif '.' in path and 'isCategory' not in page.config:
                        title = image[path]
                        image = path
                    else:
                        image = image[path]
                        if '/'in path or image is not None and '/' in image and '.com' not in image:
                            category_path = path + '/'
                            out_prefix = ''
                            if '/' in path:
                                out_prefix = ''
                                image = base_prefix + path.split('/')[-1] + '-' + image
                        else:
                            category_path = page.src_path[2:] + '/' + path + '/'
                            out_prefix = base_prefix + path + '-'
                        if image is not None and '/' in image and '.com' not in image:
                            image = image.split('/')
                            out_prefix = base_prefix + image[0] + '-'
                            image = image[1]

                        page.config['isCategory'] = True
                        page.config['categoryTitle'] = ''

                        if '.' in path and '..' not in path:
                            alt_title = ' '.join([segment.capitalize() for segment in path[22:-4].split('-')])
                            rendered_gallery += builder.interpolate(self.templates['image'], {
                                'href': 'https://' + image,
                                'src': builder.config['path'] + page.src_path[2:] + '/' + path,
                                'alt': page.config['pageTitle'] + ' - ' + alt_title,
                                'title': alt_title,
                            })
                        else:
                            category_config = builder.read_config(path=os.path.join(builder.src, category_path))
                            alt_title = category_config['title']
                            rendered_gallery += builder.interpolate(self.templates['image'], {
                                'href': builder.config['path'] + category_path,
                                'src': builder.config['path'] + category_path + 'thumb-' + out_prefix + image,
                                'alt': page.config['pageTitle'] + ' - ' + alt_title,
                                'title': alt_title,
                            })
                else:
                    name = image.split('.')[0]
                    category = page.src_path.split('/')[-2]
                    if name.isdigit() and category not in builder.config.get('gallery_autocaption_ignore', []):
                        title = page.config['title'] + ' #' + name

                if 'isCategory' not in page.config:
                    if not builder.options['remove_images'] or os.path.isfile(os.path.join(page.full_path, image)) and page.raw_config['images'].count(image) < 2:
                        if page.parent.is_index:
                            alt = ''
                        else:
                            alt = page.parent.config['pageTitle']
                        alt += ' - '
                        if title is not None:
                            alt += title
                        else:
                            alt += page.config['title']
                            title = ''
                        rendered_gallery += builder.interpolate(self.templates['image'], {
                            'href': out_prefix + image,
                            'src': 'thumb-' + out_prefix + image,
                            'alt': alt,
                            'title': title,
                        })

                        if not page.parent.is_index:
                            page.config['categoryTitle'] = ''
                        if 'childDescription' in page.parent.config:
                            page.config['description'] = builder.interpolate(page.parent.config['childDescription'], {
                                'title': page.config['title'],
                            })
                        page.config['isGallery'] = True
                    else:
                        builder.echo('Removed ' + os.path.join(page.full_path, image))
                        page.raw_config['images'].remove(original)
                        changed = True

            page.config['images'] = page.raw_config['images']
            if changed:
                with open(os.path.join(page.full_path, 'index.yaml'), 'w') as file:
                    file.write(yaml.dump(page.raw_config))
            
            if page.config['body'] != '':
                page.config['body'] = builder.interpolate(self.templates['body'], {
                    'body': page.config['body'],
                })
            body_template = 'category' if page.config.get('isCategory', False) else 'gallery'
            page.config['body'] = builder.interpolate(self.templates[body_template], {
                'title': page.config['pageTitle'],
                'body': page.config['body'],
                'images': rendered_gallery,
            })

            if page.config.get('isCategory', False) and page.web_path in builder.config['bottomMenu']:
                self.rendered_galleries[page.web_path] = page.config['body']
            elif page.config['categoryTitle'] != '':
                page.config['title'] = page.config['categoryTitle'] + ' - ' + page.config['title']
        else:
            page.config['pageTitle'] = ''
            page.config['categoryTitle'] = ''

    def render_page(self, page, builder):
        if self.rendered_gallery is None:
            self.rendered_gallery = ''.join(self.rendered_galleries[path] for path in builder.config['bottomMenu'])

        page.config['body'] = builder.interpolate(page.config['body'], {
            'gallery': self.rendered_gallery,
        })

        if 'isGallery' in page.config:
            for original_name in page.config['images']:
                if isinstance(original_name, dict):
                    original_name = list(original_name.keys())[0]
                file = os.path.join(page.dist_path, original_name)
                if 'image_prefix' in builder.config:
                    output_name = builder.config['image_prefix'] + page.dir_name + '-'+ original_name
                else:
                    output_name = original_name
                try:
                    image = Image.open(file)
                    image.thumbnail((500, 200), Image.ANTIALIAS)
                    image.save(os.path.join(page.dist_path, 'thumb-' + output_name), 'JPEG')
                    os.rename(file, os.path.join(page.dist_path, output_name))
                except KeyboardInterrupt:
                    raise
                except:
                    raise
                    builder.echo("Failed thumbnail %s: %s" % (file, sys.exc_info()[0]), error=True)





import click
import config
from build import Builder
from move import Mover

@click.group()
@click.version_option(version=config.version)
def cli():
    pass

@cli.command()
@click.argument('source')
@click.argument('dest')
@click.option(
    '--verbose/-v',
    is_flag=True,
    default=False
)
@click.option(
    '--enable-modules',
    default=''
)
@click.option(
    '--disable-modules',
    default=''
)
@click.option(
    '--discover-images/--no-discover-images', 
    default=True, 
    help='add images to a page\'s config if they are not already present'
)
@click.option(
    '--remove-images/--no-remove-images', 
    default=True, 
    help='remove images from a page\'s config if they cannot be opened'
)
def build(source, dest, **kwargs):
    try:
        builder = Builder(dist=dest, base=source, options=kwargs)
        builder.interpret()
        builder.render()
    except KeyboardInterrupt:
        click.echo('stopping...')

@cli.command()
@click.argument('image')
@click.argument('to', required=False)
def mvi(image, to):
    mover = Mover(path='.')
    mover.move(image=image, to=to)

if __name__ == '__main__':
    cli()





import os
import copy

class Page:

    src_path = None
    web_path = None
    full_path = None
    dist_path = None
    dir_name = None
    is_index = False

    parent = None
    children = None
    config = None 

    def __init__(self, src, path, builder, parent):
        self.src_path = path
        self.full_path = os.path.join(src, self.src_path)
        self.web_path = os.path.join(self.src_path[1:], '')
        self.dist_path = os.path.join(builder.dist, self.src_path)
        self.dir_name = self.src_path.split(os.sep)[-1]
        self.is_index = self.src_path == '.'

        self.children = [os.path.join(self.src_path, child) for child in os.listdir(self.full_path)]
        self.parent = parent
        self.config = builder.read_config(self.full_path)
        self.raw_config = copy.copy(self.config)





class Module:

    def __init__(self, builder):
        pass

    def interpret(self, page, builder):
        pass

    def render(self, builder):
        pass

    def render_page(self, page, builder):
        pass





import unittest
import filecmp
from stawp.build import Builder

class TestBuild(unittest.TestCase):

    def build_fixture(self, name, description):
        dest='/tmp/swp'
        source='tests/fixtures/%s/src' % name
        builder = Builder(dist=dest, base=source)
        builder.interpret()
        builder.render()
        cmp = filecmp.dircmp(dest, 'tests/fixtures/%s/dest' % name)
        for attr in ['left_only', 'right_only', 'common_funny', 'diff_files', 'funny_files']:
            self.assertEqual(len(getattr(cmp, attr)), 0, 'Failed ' + description + "\n" + attr + " - " + str(getattr(cmp, attr)))

    def test_build_min_copy(self):
        self.build_fixture('minimum-copy', 'copy file only')

    def test_build_min_page(self):
        self.build_fixture('minimum-page', 'minimum configuration with one page')

    def test_build_default(self):
        self.build_fixture('builder', 'default build behavior')





import re
import os
import sys
import yaml
import click

class Mover:

    path = None

    def __init__(self, path):
        self.path = path

    def move(self, image, to):
        config = os.path.join(self.path, 'index.yaml')
        with open(config, 'r') as file:
            page = file.read()
        page = yaml.load(page)
        images = page['images']

        if image == 'sort':
            images.sort(key=self.separate_digits)
        else:
            images.remove(image)
            if to == 'first' or to == 'front' or to == 'start':
                to = 0
            elif to == 'last' or to == 'back' or to == 'end':
                to = len(images)
            else:
                to = images.index(to)
            images.insert(to, image)

        page['images'] = images
        click.echo(page['images'])

        page = yaml.dump(page)
        with open(config, 'w') as file:
            file.write(page)

    def separate_digits(self, string):
        return [ int(char) if char.isdigit() else char for char in re.split('(\d+)', string) ]





import os
import shutil
import yaml
import imp
import click
import distutils.dir_util
from page import Page

class Builder:

    dist = None
    base = None
    src = None
    templates = None
    options = None

    config = None
    modules = None
    pages = None

    def __init__(self, dist, base, options={}):
        self.dist = dist
        self.base = base
        self.src = os.path.join(self.base, 'src')
        self.templates = os.path.join(self.base, 'html')
        self.options = options

        try:
            for path in os.listdir(self.dist):
                path = os.path.join(self.dist, path)
                if os.path.isdir(path):
                    shutil.rmtree(path)
                else:
                    os.remove(path)
        except (OSError, IOError):
            pass
        distutils.dir_util.copy_tree(self.src, self.dist)

        self.modules = []
        enabled = options.get('enable_modules', '').split(',')
        # all are disabled by default so this is just a placeholder for potential future behavior
        disabled = options.get('disable_modules', '').split(',')

        modules_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'modules')
        for module_file in os.listdir(modules_path):
            if module_file[:-3] in enabled:
                module_path = os.path.join(modules_path, module_file)
                if os.path.isfile(module_path):
                    module = imp.load_source('stawp_module_' + module_file, module_path)
                    module = module.Module(self)
                    self.modules.append(module)

    def read_template(self, path=None, name='index'):
        if path is None:
            path = self.templates
        try:
            with open(os.path.join(path, name + '.html'), 'r') as file:
                template = file.read()
            return template
        except (OSError, IOError):
            return ''

    def read_config(self, path, name='index'):
        try:
            with open(os.path.join(path, name + '.yaml'), 'r') as file:
                config = file.read()
            config = yaml.load(config)
            if config is None:
                return {}
            else:
                return config
        except (OSError, IOError):
            return None

    def interpolate(self, template, values):
        if hasattr(values, 'iteritems'):
            iterator = values.iteritems()
        else:
            iterator = values.items()
        for key, value in iterator:
            template = template.replace('{{%s}}' % key, unicode(value))
        return template

    def echo(self, message, verbose=False, error=False):
        if not verbose or self.options.get('verbose', False):
            click.echo(message, err=error)

    def interpret(self):
        self.echo('interpreting config...', verbose=True)
        self.config = self.read_config(self.base, 'config')
        if self.config is None:
            self.config = {}
        if 'path' not in self.config:
            self.config['path'] = '/'

        dirs = ['.']
        self.pages = []
        parents = {
            '.': None,
        }

        while dirs:
            next_dirs = []
            for path in dirs:
                if os.path.isdir(os.path.join(self.src, path)):
                    page = Page(src=self.src, path=path, builder=self, parent=parents[path])
                    next_dirs.extend(page.children)

                    if page.config is None:
                        continue

                    page.config['body'] = self.read_template(path=page.full_path)
                    page.config['pagePath'] = self.config.get('path', '') + path[2:]
                    if not page.is_index:
                        page.config['pagePath'] += '/'

                    for module in self.modules:
                        module.interpret(page=page, builder=self)

                    if 'description' in page.config:
                        page.config['description'] = ' ' + page.config['description']
                    else:
                        page.config['description'] = ''

                    for child in page.children:
                        parents[child] = page
                    self.pages.append(page)

            dirs = next_dirs

    def render(self):
        self.echo('rendering templates...', verbose=True)
        template = self.read_template()

        for module in self.modules:
            module.render(builder=self)

        template = self.interpolate(template, self.config)

        for page in self.pages:
            try:
                os.remove(os.path.join(page.dist_path, 'index.yaml'))
            except (OSError, IOError):
                pass

            for module in self.modules:
                module.render_page(page=page, builder=self)

            output = self.interpolate(template, page.config)
            with open(os.path.join(page.dist_path, 'index.html'), 'w') as file:
                output = output.encode('utf-8')
                file.write(output)
        self.echo('done', verbose=True)




from setuptools import setup
import importlib

config = importlib.import_module('stawp.config')

setup(
    name='stawp',
    version=config.version,
    description='STAtic Website Producer',
    author='Matt Wisniewski',
    author_email='stawp@mattw.us',
    license='MIT',
    url='https://github.com/polishmatt/stawp',
    packages=[
        'stawp',
        'stawp.modules',
    ],
    keywords=['static', 'website'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],
    install_requires=[
        'Pillow==2.3.0',
        'pyaml==15.8.2',
        'click==6.6'
    ],
    entry_points={
        'console_scripts': [
            'stawp = stawp.cli:cli'
        ],
    },
)

